/*******************************************************************/
/*                                                                 */
/*   File name :  lib_from_K5.h                                    */
/*                                                                 */
/*      Version:   2014-04-23                                      */
/*                                                                 */
/*   Copyright (c) 2014-     T.Kondo/NICT All Right Reserved       */
/*                                                                 */
/*******************************************************************/
#ifndef _LIB_FROM_K5_
#define _LIB_FROM_K5_

#include <stdio.h>
#include <signal.h> 
#include <errno.h>
#include <fcntl.h>
#include <string.h>
#include <math.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>

#ifdef __CYGWIN__
#ifndef linux
#define linux
#endif
#endif

#ifdef linux
#include <sys/time.h>
#endif

#ifndef WIN32
#include <unistd.h>
#endif

#ifdef linux
 #ifdef _K5_USE_FOPEN64_
  #ifndef _FOPEN_DEFINED_
   #define _FOPEN_DEFINED_
   #define fopen(_Z,_X) fopen64(_Z,_X)
  #endif
 #endif
 #ifdef _LARGEFILE_SOURCE
  #ifndef _FSEEK_DEFINED_
   #define _FSEEK_DEFINED_
   #define fseek(_Z,_X,_W) fseeko(_Z,_X,_W)
  #endif
  #ifndef _FTELL_DEFINED_
   #define _FTELL_DEFINED_
   #define ftell(_Z) ftello(_Z)
  #endif
 #endif
#endif

/* for WIN32 */

#ifdef WIN32
#pragma warning (disable:4996)
 #ifndef _FORWIN32SET_
  #define _FORWIN32SET_
  typedef unsigned int    u_int32_t;
  typedef unsigned int    uint32_t;
  typedef unsigned short int    u_int16_t;
  typedef unsigned short int    uint16_t;
  typedef unsigned char   u_int8_t;
  typedef _int64 int64_t;
  #define popen(_Z,_X) _popen(_Z,_X)
  #define pclose(_Z) _pclose(_Z)
  #define fseek(_Z,_X,_W) _fseeki64(_Z,_X,_W)
  #define ftell(_Z) _ftelli64(_Z)
 #endif
#endif

#ifndef USE_ASM
//#define USE_ASM
#endif


/* constant set */
#ifndef _SET_K5_CONSTANTS_
#define _SET_K5_CONSTANTS_

#define HEAD_SEARCH_RANGE 600  // header serach range (in 32bit word) -0.5*HEAD_SEARCH_RANGE to +0.5*HEAD_SEARCH_RANGE
#define VDIF_KASHIMA_SYNC 0xACABFEEDUL    // sync pattern for VDIF Kashima format

#ifndef FILEMAXSIZE
#define FILEMAXSIZE  2000000000   //  File maximum limit size in bytes when limiting
#endif

#ifndef PIRA
#define PIRA   0.017453292519943295769236907684886    /* PI/180  degree to radian */
#endif
#ifndef PIDE
#define PIDE   57.295779513082320876798154814105      /* 180/PI radian to degree  */
#endif
#ifndef TWOPI
#define TWOPI  6.283185307179586476925286766559       /* 2PI                      */
#endif
#ifndef PI
#define PI     3.1415926535897932384626433832795      /* PI                       */
#endif
#ifndef HALFPI
#define HALFPI 1.5707963267948966192313216916398      /*  PI/2                    */
#endif
#ifndef VLIGHT
#define VLIGHT 2.99792458E+8    /* speed of light (m/sec)  */
#endif

/* mainly used in libapri etc */
#ifndef CONVA
#define CONVA 4.8481368110953599358991410235795e-6    /* PI/(3600*180)  arcsec to radian */
#endif
#ifndef CONVS
#define CONVS 7.2722052166430399038487115353692e-5    /* PI/(3600*12) time sec to radian */
#endif
#ifndef CONVD
#define CONVD 0.017453292519943295769236907684886    //*PI/180  degree to radian  */
#endif
#ifndef CONVH
#define CONVH 0.26179938779914943653855361527329    /* PI/12 hour to radian  */
#endif


#define REWIND_BYTE 200       // for K5/VSSP access

#endif   // *** END of _SET_K5_CONSTANTS_


typedef struct vparam
{
    int  n;          //number of param
    char param[60][128];   // 60x128 characters
} vparam_t;

typedef struct K5VSSP_file     // originated from ip_vlbi_file type
                               // difference is displayed by **K5**
{
    /*
      Note: use of a fpos_t type variable was obsoleted because of
         the compatiblity ploblem of a fpos_t type variable between linux 2.6
         and previous one.
    */
	//unsigned char bit32[4];  // obsoleted in **K5**
	//unsigned char bit64[8]; // obsoleted in **K5**
	unsigned char *ibuf; // will be obsoleted in **K5**
	u_int32_t head32[8]; // **K5**  to contain full header including 
                         // in case of K5/VSSP32
	u_int16_t *head16; // **K5**  pointer to headder in 2 bytes unit  
	unsigned char *head8; // **K5**  pointer to headder in byte unit  
	u_int32_t *aux32; // **K5**  to contain aux field start address in
                        // head 32
	u_int32_t *ubuf; // pointer to read buffer **K5**
    int nword;       // ubuf size allocated or to be allocated  **K5**
    int auxbyte;     // # of bytes in aux field   **K5**
                     // valid byte number in aux32  (deflt=20)
    int auxbyte1st;   // # of bytes in aux field at the 1st headder  **K5**
    int aux_fmt;  // AUX field format   added on 2010-08-19
    int headsizew;    // header size in 32bit-word
    int *idata;   // pointer to decoded data (integer) container
    double *data;   // pointer to decoded data (R*8) container
	int sfreq;  //  sampling frequency in kHz
	int adbit;  // AD bits (1,2,4,8)
	int numch;  // # of channels
	int icnt;   //  current unit number in a second data 
	int numb;   //  # of bytes in unit # of samples
	int usampl;  // unit # of samples
	int kpos;  // should be obsoleted ?
	int npos;    //  valid number in idata or xdata **K5**
	int imax;    //  # of units in a second data
	int bytesec;    //  # of bytes in a second data  **K5**
	//char* fname; //pointer data file name obsoleted
	char fname[128]; //data file name opened or to be opened  **K5**
	double fsample;  // sampling frequency (Hz)
	double tsample; // sampling period (sec)
	double unitp;   // usampl*tsampl unit length 
    //long int seconds; // time (sec) from mid night (00h00m00s)
    int seconds; // time (sec) from mid night (00h00m00s)  2011-02-07
	int hh, mm, ss;  // time in hour, minutes, seconds 
    int eflg; // error flag (K5/VSSP32)  added on 2007-02-07
    int year; // 4 digits year (K5/VSSP32)
    int totalday; // (K5/VSSP32)
    int version1; // Major version (K5/VSSP32)
    int version2; // Minor version (K5/VSSP32)
	int spatn; // detected 2nd sync pattern detected 
	int syncok; // =1 for 32bit all 1 detection // not used
	int format; // =0 for old format , =1 K5/VSSP, =2 K5/VSSP32
                //         =2 is added **K5**
	FILE *stream;
	//fpos_t pos;   // changed to int64_t type 
	//fpos_t size;  // changed to int64_t type
                    // because fpos_t type is no more scalar value in linux 2.6
	int64_t pos;   // current pointer
	int64_t size;  // opened file size in bytes **K5**
	int fh;
	int ch;    
	int ierr;   // error code (general purpose)
	double dtime;  // time at begin of unit
	double dtimest;  // time at begin of file data
	unsigned int bytesread;  // number of bytes read
	int64_t tbytesread;   // total # of bytes read **K5**
	int wordread;   // # of 32bit-words read counter (general) **K5**
	int64_t twordread;   // total # of 32bit-word read **K5**
    int vflag, vflag_old;  // byte slip flag  added on 2003-02-14
                           //  if header sync detection is suceed
                           //     set to 1.  else 0
                           //   1: sync detection success 
                           //   0: sync detection error (may be byte slip)
                           //   vflag_old is the previous status
    //long int dtime_vflag, dtime_vflag_old; // time at vflag decision made
    int dtime_vflag, dtime_vflag_old; // time at vflag decision made  2011-02-07
	// add 2007-03-02 for robust sync pattern dfinding
	//      sampling parameters and time at 1 st frame
	int sfreq1st;  //  sampling frequency in kHz
	int adbit1st;  // AD bits (1,2,4,8)
	int numch1st;  // # of channels
    //long int seconds1st; // time (sec) from mid night (00h00m00s)
    //long int secold; // time (sec) from mid night (00h00m00s)
    long int seconds1st; // time (sec) from mid night (00h00m00s)  2011-02-07
    long int secold; // time (sec) from mid night (00h00m00s)      2011-02-07
	int framenum;  // frame number starting from 0

    int flag_fileclose;  // 2014-06-11   only for libvdif siries 
} K5VSSP_file_t;

typedef struct vsspaux  // VSSP32 AUX field type  for K5/VSSP32  2005-12-28 (moved from libipvlbi_board.h)
{
    int size;    // AUX filed size
    int fmt;     // AUX field format
    int filter;  // LPF filter frequency(MHz) for K5/VSSP32 (0: through)
    char bdat[256];   // AUX field data
    char sid[2];        // 1 char Station ID (no more use)
    char sid2[3];       // 2 char Station ID
    char station8[20];       // station name (1st 8 char is valid)
    char hostname8[20];       // Host PC name (1st 8 char is valid)
} vsspaux_t;



#ifdef __cplusplus
extern "C"{
#endif 

void decode_data(int *K5pos, int wd, int ilast, int pat, int adbit);

/*** copy from libipvlbi.h and libmk5tok5.h****/
int rvsint(int i4);
void read1line(char *line);
int64_t get_file_size(char *fname);
void conv2small(char *cbuf);
void conv2large(char *cbuf);
int cmnd_get_p(int argc, char *argv[], char *cmnd, char *param);  // 2014-05-22
int cmnd_get_n2(int argc, char *argv[], char *cmnd, char *param, char *get_cmnd);
int cmnd_get_n(int argc, char *argv[], char *cmnd, char *param);
int cmnd_get(int argc, char *argv[], char *command, char *param);
char *select_env_dflt(char *env, char *dflt);
void parse_dir_and_name(char *fpath, char *dir, char *fname);
int substitute_env_path_name(char *path, char *fpath);
void get_true_path(char *path, char *dfltenv, char *dfltdir2, char *fpath);
void k5name_make(char *mk5name, char *outdir, int channel, int group, 
                 char *k5name);
void parse_dir_and_name_fm_fpath(char *fpath, char *dir, char *fname) ;
void sec2hms(int sec, int* hh, int* mm, int* ss);

/********** copy from libvssp32.h ********/
void k5vssp_t_ini(K5VSSP_file_t *K5);
int CheckK5header_with_Read(K5VSSP_file_t *S);
int fwdNsecK5(int nsec, K5VSSP_file_t *K5);
int fwdNsecK5org(int nsec, K5VSSP_file_t *K5);
int CheckK5header_with_Read_but_no_search(K5VSSP_file_t *S);
int CheckK5header(K5VSSP_file_t *K5);
int CheckK5header_with_Read2(K5VSSP_file_t *S, int maxbytes);
int header_aux_read(K5VSSP_file_t *K5);
void vssp_aux_parse(vsspaux_t *vsspaux);

#ifdef __cplusplus
}
#endif

#endif
